//=====================================================================================================================
/// @file		RFM95W.c
///
/// @brief		Variables and Functions for RFM95W
///
/// @version	V1.0
///
/// @date		01.08.2020
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "RFM95W.h"

//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function: Rfm95WriteReg =============================================================================================
///
/// @brief	Write to RFM95W Registers
///
/// @param 	uint8_t adress, uint8_t value
///
/// @return	bool
//=====================================================================================================================
bool Rfm95WriteReg(uint8_t adress, uint8_t value)
{
	//Check Variable
	bool result = true;
	//Add Write Command to Adress-Byte
	uint8_t adrWrite = (adress | (1<<7));
	//Data array for Transmission
	uint8_t writeReg[2] = {adrWrite, value};
	//Dummy Array for Answer
	uint8_t dummy[2];
	//Send Write Command
	SpiTransceiveMaster(&writeReg[0], &dummy[0], 2 );
	return result;

}
//=== Function: Rfm95ReadReg =============================================================================================
///
/// @brief	Read RFM95W Registers
///
/// @param 	uint8_t adress, uint8_t *firstElementOfData
///
/// @return	bool
//=====================================================================================================================
bool Rfm95ReadReg(uint8_t adress, uint8_t *firstElementOfData)
{
	//Check Variable
	bool result = true;
	//Data Array for Transmission
	uint8_t readReg[2] = {adress, 0x00};
	//Data Array for received Data
	uint8_t receivedRegData[2];
	//Send read Command and receive Answer
	SpiTransceiveMaster(&readReg[0], &receivedRegData[0], 2 );
	//Return Answer
	*firstElementOfData = receivedRegData[1];
	//Return Check Variable
	return result;
}
//=== Function: Rfm95Init =============================================================================================
///
/// @brief	Initialize RFM95W and Switch to Standby Mode
///
/// @param float carFreq, uint8_t signalBw, uint8_t codRate, uint8_t spreadFactor, uint8_t crcOn
///
/// @return	bool
//=====================================================================================================================
bool Rfm95Init(float carFreq, uint8_t signalBw, uint8_t codRate, uint8_t spreadFactor, uint8_t crcOn, uint8_t preambleLen, uint8_t syncWord)
{
	//Check Variable
	bool result = true;
	//Previous and new Register entry for LoRa Mode
	uint8_t prevRegLora;
	uint8_t newRegLora;
	//Register Entry for Carrier Frequency = desired Frequency / ((FXOSC=32MHz)/(2^19))
	uint32_t carFreqReg = carFreq / 61.03515625 ;
	//Switch to SLEEP Mode
	Rfm95ChangeMode(0x00);
	//Switch to Lora Mode
	Rfm95ReadReg(0x01, &prevRegLora);
	newRegLora = prevRegLora |  0x80;
	Rfm95WriteReg(0x01, newRegLora);
	//Write Register Entry for carrier Frequency
	Rfm95WriteReg(0x06, (carFreqReg>>16));
	Rfm95WriteReg(0x07, (carFreqReg>>8));
	Rfm95WriteReg(0x08, carFreqReg);
	//Write Register Entry for Bandwidth, Coding Rate and Explicit Header
	Rfm95WriteReg(0x1D,((signalBw<<4) | (codRate<<1)));
	//Write Registry Entry for Spreading Factor and CRC
	Rfm95WriteReg(0x1E, ((spreadFactor<<4) | (crcOn<<2)));
	//Write Registry Entry for Preamble Length
	Rfm95WriteReg(0x21, preambleLen);
	//Write Registry Entry for Sync Word
	Rfm95WriteReg(0x39, syncWord);
	
	/*
	//Switch to Receive-Mode to get RSSI Measurement
	Rfm95ChangeMode(5); 
	//Write RSSI Measurement to devNonce (pseudo random)
	Rfm95ReadReg(0x1B, &devNonce[0]);
	Rfm95ReadReg(0x1B, &devNonce[1]);
	*/
	
	devNonce[0] = Rfm95GetRandomNumber();
	devNonce[1] = Rfm95GetRandomNumber();
	
	//Switch to Sleep Mode
	Rfm95ChangeMode(0);
	return result;
}
//=== Function: Rfm95ChangeMode =============================================================================================
///
/// @brief	Switch operating Mode of RFM95W
///
/// @param uint8_t mode // 0=SLEEP, 1=STDBY, 2=FSTX, 3=TX, 4=FSRX, 5=RXCONTINUOUS, 6=RXSINGLE, 7=CAD
///
/// @return	bool
//=====================================================================================================================
bool Rfm95ChangeMode(uint8_t mode)
{
	//Check Variable
	bool result = true;
	//Variable for previous OpModReg entry
	uint8_t prevMode = 0;
	//Variable for new OpModReg entry
	uint8_t newMode = 0;
	//Query previous OpModReg entry
	Rfm95ReadReg(0x01, &prevMode);
	//Calculate new OpModReg entry
	newMode = (prevMode & 0xF8) | (mode);
	//Write new OpModReg entry, query until new Mode is active
	Rfm95WriteReg(0x01, newMode);
	while(prevMode != newMode)
	{
		Rfm95ReadReg(0x01, &prevMode);
	}
	//Return Check Variable
	return result;
}
//=== Function: Rfm95Tx =============================================================================================
///
/// @brief	Transmit Data Packet via LoRa
///
/// @param uint8_t *firstElementOfData, uint8_t numberOfElements //127byte max
///
/// @return	bool
//=====================================================================================================================
bool Rfm95Tx(uint8_t *firstElementOfData, uint8_t numberOfElements)
{
	//Check Variable
	bool result = true;
	//Loop Variable
	uint8_t i;
	//Base Adress of FIFO for TX
	uint8_t baseAdressTx;
	//TxDone IRQ Flag
	uint8_t txDoneFlag;
	if ((0 < numberOfElements) & (numberOfElements < 129))
	{
		//Switch to Standby Mode
		Rfm95ChangeMode(1);
		//Get Base Adress of FIFO for TX
		Rfm95ReadReg(0x0E, &baseAdressTx);
		//Set FIFO Adress Pointer to Base Adress for TX
		Rfm95WriteReg(0x0D, baseAdressTx);
		//Write Payload Length in Byte to RegPayloadLength
		Rfm95WriteReg(0x22, numberOfElements);
		//Write Data to FIFO// AdressPointer is incremented automatically
		for(i=0; i<numberOfElements;i++)
		{
			Rfm95WriteReg(0x00,*firstElementOfData);
			firstElementOfData++;
		}
		//Reset Data Pointer
		firstElementOfData -= i;
		//Switch to TX Mode
		Rfm95ChangeMode(3);
		//Wait for IRQ TxDone
		while (txDoneFlag != 1)
		{
			Rfm95ReadReg(0x12, &txDoneFlag);
			i = CHECK_REG_BIT(txDoneFlag, 3);
			txDoneFlag = i;
		}
		//Clear IRQ TxDone
		Rfm95WriteReg(0x12, 0x08);
	}
	else
	{
		result = false;
	}
	//Switch to SLEEP Mode
	Rfm95ChangeMode(0);
	//Return Check Variable
	return result;
}
//=== Function: Rfm95Rx =============================================================================================
///
/// @brief	Receive Data Packet via LoRa in Single Reception Mode (specific time window)
///
/// @param uint8_t *firstElementOfData, uint8_t timeout , uint8_t *numberOfRxBytes
///
/// @return	bool // false = timeout
//=====================================================================================================================
bool Rfm95Rx(uint8_t *firstElementofData, uint8_t timeout, uint8_t *numberOfRxBytes)
{
	//Check Variable
	bool result = true;
	//Auxiliary Variables
	uint8_t i;
	uint8_t j;
	//Base Adress of FIFO for Rx
	uint8_t baseAdressRx;
	//RxDone IRQ Flag
	uint8_t rxDoneFlag = 0;
	//RxTimeout IRQ Flag
	uint8_t rxTimeoutFlag = 0;
	//PayloadCrcError and ValidHeader IRQ Flags
	uint8_t rxCrcHeader = 0;
	//Number of received Bytes
	uint8_t rxNumberOfBytes;
	//Start adress of current Packet in FIFO
	uint8_t rxCurrentAdress;
	//Switch to Standby Mode
	Rfm95ChangeMode(1);
	//Timeout time in Signal Periods
	Rfm95WriteReg(0x1F, timeout);
	//Get Base Adress of FIFO for RX
	Rfm95ReadReg(0x0F, &baseAdressRx);
	//Set FIFO Adress Pointer to Base Adress for RX
	Rfm95WriteReg(0x0D, baseAdressRx);
	//Start Rx by switching to RXSINGLE mode
	Rfm95ChangeMode(6);
	//Wait for RxDone IRQ or rxTimeout IRQ
	while (rxDoneFlag != 1 && rxTimeoutFlag != 1)
	{
		Rfm95ReadReg(0x12, &rxDoneFlag);
		i = CHECK_REG_BIT(rxDoneFlag, 6);
		rxDoneFlag = i;
		Rfm95ReadReg(0x12, &rxTimeoutFlag);
		j = CHECK_REG_BIT(rxTimeoutFlag, 7);
		rxTimeoutFlag = j;
	}
	//Check for RxDone IRQ
	if (rxDoneFlag == 1)
	{
		//Check for CRC error and Valid Header
		Rfm95ReadReg(0x12, &rxCrcHeader);
		i = rxCrcHeader & 0x30;
		rxCrcHeader = i;
		if (rxCrcHeader == 0x10)
		{
			//Clear rxDoneFlag
			Rfm95WriteReg(0x12, 0x40);
			//Read Number of received bytes
			Rfm95ReadReg(0x13, &rxNumberOfBytes);
			*numberOfRxBytes = rxNumberOfBytes;			
			//Read start adress of current packet in FIFO
			Rfm95ReadReg(0x10, &rxCurrentAdress);
			//Set FIFO address Pointer to start adress of current packet
			Rfm95WriteReg(0x0D, rxCurrentAdress);
			//Read received Package
			for (i=0; i < rxNumberOfBytes; i++)
			{
				Rfm95ReadReg(0x00, (firstElementofData+i));
				//firstElementofData++;
			}
			//Reset Data Pointer
			//firstElementofData -= i;
			result = true;
		}
		else
		{
			//Clear rxDoneFlag & rxCrcHeader
			Rfm95WriteReg(0x12, 0x70);
			result = false;
		}
	}
	//Check for RxTimeout IRQ
	else if (rxTimeoutFlag == 1)
	{
		//Clear rxTimeout IRQ
		Rfm95WriteReg(0x12, 0x80);
		result = false;
	}
	else
	{
		result = false;
	}
	//Switch to Sleep mode
	Rfm95ChangeMode(0);
	//Return check variable
	return result;
}
//=== Function: Rfm95ChangeChangeCarFreq =============================================================================================
///
/// @brief	Change Carrier Frequency
///
/// @param float carFreq (in Hz)
///
/// @return	bool
//=====================================================================================================================
bool Rfm95ChangeCarFreq(float carFreq)
{
	//Check variable
	bool result = true;
	//Register Entry for Carrier Frequency = desired Frequency / ((FXOSC=32MHz)/(2^19))
	uint32_t carFreqReg = carFreq / 61.03515625 ;
	//Write Register Entry for carrier Frequency
	Rfm95WriteReg(0x06, (carFreqReg>>16));
	Rfm95WriteReg(0x07, (carFreqReg>>8));
	Rfm95WriteReg(0x08, carFreqReg);
	return result;
}
//=== Function: Rfm95ChangeChangeBandwidth =============================================================================================
///
/// @brief	Change Bandwidth of Chirp Spread Spectrum	
///
/// @param uint8_t bandwidth //0=7.8kHz, 1=10.4kHz, 2=15.6kHz, 3=20.8kHz, 4=31.25kHz, 5=41.7kHz, 6=62.5kHz, 7=125kHz, 8=250kHz, 9=500kHz
///
/// @return	bool
//=====================================================================================================================
bool Rfm95ChangeBandwidth (uint8_t bandwidth)
{
	//check variable
	bool result = true;
	//variable for previous registry entry
	uint8_t previous;
	//variable for new registry entry
	uint8_t new;
	//Read Presvious entry
	Rfm95ReadReg(0x1D, &previous);
	//Calculate new entry
	new = (previous&0x0F)|(bandwidth<<4);
	//Write new entry
	Rfm95WriteReg(0x1D, new);
	 return result;
}
//=== Function: Rfm95ChangeSpreadFactor =============================================================================================
///
/// @brief	Change Spread Factor
///
/// @param uint8_t spreadFactor // Chips/Symbol = (SF^2)
///
/// @return	bool
//=====================================================================================================================
bool Rfm95ChangeSpreadFactor (uint8_t spreadFactor)
{
	//check variable
	bool result = true;
	//variable for previous registry entry
	uint8_t previous;
	//variable for new registry entry
	uint8_t new;
	//Read Presvious entry
	Rfm95ReadReg(0x1E, &previous);
	//Calculate new entry
	new = (previous&0x0F)|(spreadFactor<<4);
	//Write new entry
	Rfm95WriteReg(0x1E, new);
	return result;
	
}
//=== Function: Rfm95GetRandomNumber =============================================================================================
///
/// @brief	Gets pseudo random number from wideband RSSI measurement
///
/// @param 	void
///
/// @return	uint8_t
//=====================================================================================================================
uint8_t Rfm95GetRandomNumber ()
{
	uint8_t result;
	Rfm95ChangeMode(5);
	Rfm95ReadReg(0x2C, &result);
	Rfm95ChangeMode(0);
	return result;
};
/*
//=== Function: SI_INTERRUPT(TIMER0_ISR, TIMER0_IRQn) =================================================================
///
/// @brief	
///
/// @param
///
/// @return
//=====================================================================================================================
SI_INTERRUPT(TIMER0_ISR, TIMER0_IRQn)
{
}
*/