//=====================================================================================================================
/// @file		SPI_ATmega328P.c
///
/// @brief		Variables and Functions for SPI Interface of ATmega328p
///
/// @version	V1.0
///
/// @date		01.08.2020
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "SPI_ATmega328p.h"

//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function: SpiInitMaster =============================================================================================
///
/// @brief	Initialize SPI Interface (Master)
///
/// @param 	void
///
/// @return	void
//=====================================================================================================================
void SpiInitMaster(void)
{
	//PB2(SS) to Output
	SET_REG_BIT(DDRB, DDB2);
	//SS HIGH
	SET_REG_BIT(PORTB, PORTB2);
	//PB5(SCK) to Output
	SET_REG_BIT(DDRB, DDB5);
	//PB3(MOSI) to Output
	SET_REG_BIT(DDRB, DDB3);
	//Set SPI to Master Mode
	SET_REG_BIT(SPCR, MSTR);
	//Enable SPI
	SET_REG_BIT(SPCR, SPE);
}
/*
//=== Function: SpiInitSlave =============================================================================================
///
/// @brief	Initialize SPI Interface (Slave)
///
/// @param 	void
///
/// @return	void
//=====================================================================================================================
void SpiInitSlave(void)
{
	//Set PB4(MISO) to Output
	SET_REG_BIT(DDRB, DDB4);
	//Enable SPI
	SET_REG_BIT(SPCR, SPE);
}
*/
//=== Function: SpiTransceiveMaster =====================================================================================
///
/// @brief	Transceive Data Array via SPI (Master)
///
/// @param	uint8_t *firstElementOfSendData, uint8_t *firstElementOfRecData, uint8_t numberOfElements
///
/// @return	bool
//=====================================================================================================================
bool SpiTransceiveMaster(uint8_t *firstElementOfSendData, uint8_t *firstElementOfRecData, uint8_t numberOfElements)
{
	//Check Variable
	bool result = true;
	//Loop Variable
	uint8_t i = 0;
	//Transmission if Number of Data Elements > 0
	if (numberOfElements > 0)
	{
		//SS LOW
		DEL_REG_BIT(PORTB, PORTB2);
		for (i=0;i<numberOfElements;i++)
		{
			//Write Data to SPI Data Register
			SPDR = *firstElementOfSendData;
			//Wait for SPI Flag = 1
			while(!(SPSR & (1<<SPIF)));
			//Read received Data from SPI Data Register
			*firstElementOfRecData = SPDR;
			//Incrementation of Pointers
			firstElementOfSendData++;
			firstElementOfRecData++;
		}
		//Pull SS HIGH
		SET_REG_BIT(PORTB, PORTB2);
		//Reset Pointers
		firstElementOfSendData -= i;
		firstElementOfRecData -= i;
	}
	//Return of Check Variable
	return result;
}
/*
//=== Function: SpiTransceiveSlave =====================================================================================
///
/// @brief	Transceive Data Array via SPI (Slave)
///
/// @param	uint8_t *sendData, uint8_t recData
///
/// @return	bool
//=====================================================================================================================
bool SpiTransceiveSlave(uint8_t *sendData, uint8_t *recData)
{
	//Check Variable
	bool result = true;
	
	*recData = SPDR;
	SPDR = *sendData;
	//Return Check Variable
	return result;
}
*/
/*
//=== Function: SI_INTERRUPT(TIMER0_ISR, TIMER0_IRQn) =================================================================
///
/// @brief	
///
/// @param
///
/// @return
//=====================================================================================================================
SI_INTERRUPT(TIMER0_ISR, TIMER0_IRQn)
{
}
*/