//=====================================================================================================================
/// @file		UART_atmega328p.c
///
/// @brief		Variables and Functions for UART Interface of ATmega328p
///
/// @version	V1.0
///
/// @date		01.12.20
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "UART_atmega328p.h"
//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function:	UartInit =============================================================================================
///
/// @brief	Initialize UART Interface
///
/// @param	uint8_t clockFreq, uint16_t baud
///
/// @return
//=====================================================================================================================
void UartInit(uint8_t clockFreq, uint32_t baud)
{
	//Baudrate register entry
	long baudReg;
	//Set Frame Size to 8 bit
	//UCSR0C |= (1<<UCSZ00);
	//UCSR0C |= (1<<UCSZ01);
	//Set Baudrate
	baudReg = (((clockFreq*1000000)/(16*baud))-1);
	UBRR0L = baudReg;
	UBRR0H = (baudReg>>8);
	//Enable USART TX and RX
	UCSR0B |= (1<<TXEN0);
	UCSR0B |= (1<<RXEN0);
};
//=== Function:	UartTransmit =============================================================================================
///
/// @brief	Transmit Data Array via UART Interface
///
/// @param  uint8_t *firstElementOfData, uint8_t lengthOfData
///
/// @return
//=====================================================================================================================
void UartTransmit (uint8_t *firstElementOfData, uint8_t lengthOfData)
{
	//Loop variable
	uint8_t i;
	//Wait for ready to write flag and write to data register
	for (i=0;i<lengthOfData;i++)
	{
		while (!(UCSR0A&(1<<UDRE0))){}
		UDR0 = *(firstElementOfData+i);
	}


}
//=== Function:	UartPrintText =============================================================================================
///
/// @brief	Print Text via UART Interface
///
/// @param	char output[] 
///
/// @return
//=====================================================================================================================
void UartPrintText (char output[])
{
	//Loop variable
	uint8_t i;
	//Length of String
	uint8_t size = strlen(output);
	//ASCII value for Space
	uint8_t space = 0x20;
	//Array for ASCII values
	uint8_t outputUart[size];
	//Convert String to ASCII Values
	for (i=0;i<size;i++)
	{
		outputUart[i] = output[i];
	}
	//Transmit ASCII values via UART
	UartTransmit(&outputUart[0], size);
	//Tansmit ASCII value for Space
	UartTransmit(&space,1);
	
}
//=== Function:	UartPrintData =============================================================================================
///
/// @brief	Print Data in HEX format in ASCII Code via UART Interface
///
/// @param	uint8_t *firstElementOfData, uint8_t lengthOfData
///
/// @return
//=====================================================================================================================
void UartPrintData (uint8_t *firstElementOfData, uint8_t lengthOfData)
{
	//Loop variable
	uint8_t i;
	//Lookup table with characters for hex values
	char charTable[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	//Output string 
	char charOutput[5];
	charOutput[4] = '\0'; 
	charOutput[0] = '0';
	charOutput[1] = 'x';
	//Write characters corresponding to hex values to String and transmit via UART
	for (i=0;i<lengthOfData;i++)
	{
		charOutput[3] = charTable[(*(firstElementOfData+i) & 0x0F)];
		charOutput[2] = charTable[(*(firstElementOfData+i) >> 0x04)];
		UartPrintText(charOutput);
	}
}

//=== Function:	UartPrintLine =============================================================================================
///
/// @brief	Transmit new line ASCII value via UART Interface
///
/// @param
///
/// @return
//=====================================================================================================================
void UartPrintNewLine()
{
	//ASCII Value for new line
	uint8_t newLine = 0x0A;
	//Transmit ASCII value for new line
	UartTransmit(&newLine,1);
}
//=== Function:	UartPrintLine =============================================================================================
///
/// @brief	Print Text and Data in HEX format via UART Interface
///
/// @param	char text[], uint8_t *data, uint8_t dataSize
///
/// @return
//=====================================================================================================================
void UartPrintLine(char text[], uint8_t *data, uint8_t dataSize)
{
	UartPrintText(text);
	UartPrintData(data, dataSize);
	UartPrintNewLine();
}