//=====================================================================================================================
/// @file		atmega328p.c
///
/// @brief		Variables and functions for using features of the Atmel ATmega328p MCU
///
/// @version	V1.0
///
/// @date		05.11.20
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "atmega328p.h"
//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------
//Counter steps for 1 ms
uint8_t counterSteps;
//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function: atmega328pInit =============================================================================================
///
/// @brief	Initialize ATmega328p MCU
///
/// @param
///
/// @return
//=====================================================================================================================
void Atmega328pInit(uint8_t clockFreq)
{
	
	Atmega328pTimerInit(clockFreq);


}
//=== Function: atmega328pTimerInit =============================================================================================
///
/// @brief	Initialize ATmega328p Timer0
///
/// @param
///
/// @return
//=====================================================================================================================
void Atmega328pTimerInit(uint8_t clockFreq)
{	
	//Set Timer0 to CTC mode
	TCCR0A |= 0x02;
	//Set Timer 0 Prescaler to 256; Start Timer
	TCCR0B |= 0x04;
	//Determine Counter Steps for ~1ms
	counterSteps = clockFreq*3.90625;
	//Set compare Value for Timer0 to corresponding value
	OCR0A = counterSteps;
}
//=== Function: atmega328pDelay =============================================================================================
///
/// @brief	Delay
///
/// @param uint16_t msDelay //Delay time in ms
///
/// @return
//=====================================================================================================================
void Atmega328pDelay(uint16_t msDelay)
{
	uint16_t msCounter = 0;
	TCNT0 = 0;
	while (msCounter<=msDelay)
	{
		if ((TIFR0 & (1<<OCF0A)) > 0)
		{
			msCounter++;
			TIFR0 |= (1<<OCF0A);
		}
	}
}
//=== Function: atmega328pStopWatchStart =============================================================================================
///
/// @brief	Start Stopwatch
///
/// @param uint16_t msDelay // Max. 1048ms
///
/// @return
//=====================================================================================================================
void Atmega328pStopWatchStart(uint16_t msDelay)
{
	//Set Compare Match for Timer 
	uint16_t compareMatch;
	compareMatch = (msDelay*counterSteps);
	//Write Value for Compare Match to Register
	OCR1AH = (compareMatch>>8);
	OCR1AL = (compareMatch);
	//Clear Timer Register
	TCNT1H = 0;
	TCNT1L = 0;
	//Set Timer 1 Prescaler to 256; Start Timer
	TCCR1B |= 0x04;

}
//=== Function: atmega328pStopWatchCheck =============================================================================================
///
/// @brief	Check Stopwatch
///
/// @param 
///
/// @return
//=====================================================================================================================
bool Atmega328pStopWatchCheck()
{
	//Check Variable
	bool result = false;
	//Check compare flag
	if ((TIFR1 & (1<<OCF1A)) > 0 )
	{
		result = true;
		//Deactivate prescaler/timer
		TCCR1B = 0;
		//Delete compare flag
		TIFR1 |= (1<<OCF1A);

		
	}
	return result;
}
/*
ISR(TIMER0_COMPA_vect)
{
	msCounter++;
}
*/