//=====================================================================================================================
/// @file		lorawan.c
///
/// @brief		Variables and functions for transceiving data via LoRaWAN protocol 
///
/// @version	V1.0
///
/// @date		01.09.20
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "lorawan.h"
//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------

//Session variables
bool activeSession = false;

//Network Identifiers and Encryption Keys
//Device EUI (little endian)
static uint8_t devEui[] = { 0x82, 0xA2, 0xF0, 0x79, 0x27, 0xB0, 0x92, 0x00 };
//Application EUI (little endian)
static uint8_t appEui[] = { 0xCC, 0x85, 0x03, 0xD0, 0x7E, 0xD5, 0xB3, 0x70 };
//Application Key (big endian)
static uint8_t appKey[] = {0xFB,0x1C,0xA3,0xC0,0x10,0xFC,0x45,0x09,0x1A,0x23,0x3A,0x68,0x84,0xA0,0x62,0xE5};
//Network Session Key (big endian)
static uint8_t nwkSKey[16];
//Application Session Key (big endian)
static uint8_t appSKey[16];
//Device Adress (little endian)
static uint8_t devAdress[4];
//Network ID (little endian)
static uint8_t netId[3];

//Channel settings and timing parameters
float channelList[NUMBER_OF_CHANNELS] = {CHANNEL_LIST};
float rx2Channel = RX2_FREQUENCY;
uint8_t dataRates[NUMBER_OF_DATA_RATES*2] = {DATA_RATES};
uint8_t rx2DataRate = DR_RX2;
uint8_t currentDataRate = (NUMBER_OF_DATA_RATES-1);
uint8_t currentChannel = 1;
uint16_t receiveDelay_1 = 1; //in seconds default: 1
uint16_t joinAcceptDelay_1 = 5;
uint16_t reTransBackOff;//Retransmission back-off in seconds
uint8_t dlSettings;
uint8_t cFList[16];
	
//Frame Parameters
//Frame counter up
uint32_t fCntUp;
//Frame counter down
uint32_t fCntDown;
//Max gap between frame counters
uint16_t maxFcntGap = 16384;
//Adaptive Data Rate
bool adr = ADR_DEFAULT;
//Acknowledgment for last uplink pending
bool ackPendingUp = false;
//Acknowledgment for last downlink pending
bool ackPendingDown = false;
//MAC Command in Queue
bool macPending = false;
uint8_t macPendingCommand[15];
uint8_t macPendingLength;
//Another Frame form Server pending, Server request for immediate uplink
bool fPending;



//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function: LorawanChangeDr =============================================================================================
///
/// @brief	Change Datarate DR according to LoRaWAN Standard
///
/// @param	uint8_t dataRate
///
/// @return 
//=====================================================================================================================
void LorawanChangeDr(uint8_t dataRate)
{

	//Change Spreadfactor
	Rfm95ChangeSpreadFactor(dataRates[dataRate*2]);
	//Change Bandwidth
	Rfm95ChangeBandwidth(dataRates[(dataRate*2)+1]);
}
//=== Function: LorawanChangeChannel =============================================================================================
///
/// @brief	Change Channel according to LoRaWAN Standard
///
/// @param	uint8_t channel
///
/// @return
//=====================================================================================================================
void LorawanChangeChannel(uint8_t channel)
{
	Rfm95ChangeCarFreq(channelList[channel]);
}
//=== Function: LorawanCreateJoinRequest =============================================================================================
///
/// @brief	Create 23-Byte long Join-request Message
///
/// @param uint8_t *firstElementOfOutput
///
/// @return
//=====================================================================================================================
bool LorawanCreateJoinRequest(uint8_t *firstElementOfOutput)
{
	//Check Variable
	bool result = true;
	//Counter variables
	uint8_t i = 0; 
	uint8_t j = 0;
	uint8_t k = 0;
	//Array for Join-request Message
	uint8_t joinRequestMessage[23];
	//Array for MIC
	uint8_t joinMic[4];
	//First Byte Containing MHDR has value 0
	joinRequestMessage[0] = 0;
	i = i+1;
	//Write AppEUI to Join-request Message
	k=i+8;
	j = 0;
	while (i<k)
	{
		joinRequestMessage[i] = appEui[j];
		i++;
		j++;
	}
	//Write DevEUI to Join-request Message
	k=i+8;
	j = 0;
	while (i<k)
	{
		joinRequestMessage[i] = devEui[j];
		i++;
		j++;
	}
	//Write DevNonce to Join-request Message
	k=i+2;
	j = 0;
	while (i<k)
	{
		joinRequestMessage[i] = devNonce[j];
		i++;
		j++;
	}
	LorawanJoinMicCalc(&joinRequestMessage[0], &joinMic[0]);
	//Write MIC to Join-request Message
	k=i+4;
	j = 0;
	while (i<k)
	{
		joinRequestMessage[i] = joinMic[j];
		i++;
		j++;
	}
	//Output joinRequestMessage
	j = 0;
	while (j<i)
	{
		*(firstElementOfOutput+j) = joinRequestMessage[j];
		j++;
	}
	return result;
}
//=== Function: LorawanJoinMicCalc =============================================================================================
///
/// @brief	Calculate MIC (Message Integrity Code) for Join Request via CMAC encryption
///
/// @param	uint8_t *firstElementOfData, uint8_t *micOutput
///
/// @return bool
//=====================================================================================================================
bool LorawanJoinMicCalc (uint8_t *firstElementOfData, uint8_t *micOutput)
{
	//Check Variable
	bool result = true;
	//Calculated MIC field
	uint8_t mic[16];
	//CMAC Context Variable
	AES_CMAC_CTX aesCmacCtx[1];
	//Initialize and Execute CMAC
	AES_CMAC_Init(aesCmacCtx);
	AES_CMAC_SetKey(aesCmacCtx, appKey);
	AES_CMAC_Update(aesCmacCtx, firstElementOfData, 19);
	AES_CMAC_Final(mic, aesCmacCtx);
	*micOutput = mic[0];
	*(micOutput+1) = mic[1];
	*(micOutput+2) = mic[2];
	*(micOutput+3) = mic[3];
	return result;
}
//=== Function: LorawanJoinDecrypt =============================================================================================
///
/// @brief	Decrypt LoraWan Join Accept using AES128
///
/// @param	uint8_t *firstElementOfData, uint8_t *Output
///
/// @return bool
//=====================================================================================================================
bool LorawanJoinDecrypt (uint8_t *firstElementOfData, uint8_t lengthOfMessage)
{
	//Check variable
	bool result = true;
	//Loop variable
	uint8_t i = 0;
	//Encrypted Join Accept Message
	uint8_t joinAcceptMessageEnc[lengthOfMessage];
	//Decrypted Join Accept Message
	uint8_t joinAcceptMessage[lengthOfMessage];
	//Array for NwksKey and AppSKey calculation
	uint8_t cryptbox[16];
	//AES Context Variable
	aes_context aesContext;
	memset(aesContext.ksch, '\0', 240);
	//Set Key in Context Variable
	aes_set_key(appKey, 16, &aesContext);
	//Write Encrypted Join Accept Message in Array
	for (i=0; i<(lengthOfMessage); i++)
	{
		joinAcceptMessageEnc[i] = *(firstElementOfData+i);
	}
	//Decrypt Join Accept Message
	aes_encrypt(joinAcceptMessageEnc, joinAcceptMessage, &aesContext);
	//Get NetId, Device Adress and rxDelay fron Join Accept Message
	netId[0] = joinAcceptMessage[3];
	netId[1] = joinAcceptMessage[4];
	netId[2] = joinAcceptMessage[5];
	devAdress[0] = joinAcceptMessage[6];
	devAdress[1] = joinAcceptMessage[7];
	devAdress[2] = joinAcceptMessage[8];
	devAdress[3] = joinAcceptMessage[9];
	dlSettings = joinAcceptMessage[10];
	receiveDelay_1 = joinAcceptMessage [11];
	//Get optional Channel Frequency List
	if (lengthOfMessage > 12)
	{
		for (i=0; i<16; i++)
		{
			cFList[i] = joinAcceptMessage[i+12];
		}
	}
	//Write Array for NwkSKey and AppSkey decryption
	cryptbox[0] = 0x01;
	cryptbox[1] = joinAcceptMessage[0];
	cryptbox[2] = joinAcceptMessage[1];
	cryptbox[3] = joinAcceptMessage[2];
	cryptbox[4] = netId[0];
	cryptbox[5] = netId[1];
	cryptbox[6] = netId[2];
	cryptbox[7] = devNonce[0];
	cryptbox[8] = devNonce[1];
	cryptbox[9] = 0x00;
	cryptbox[10] = 0x00;
	cryptbox[11] = 0x00;
	cryptbox[12] = 0x00;
	cryptbox[13] = 0x00;
	cryptbox[14] = 0x00;
	cryptbox[15] = 0x00;
	cryptbox[16] = 0x00;
	//Decrypt nwkSKey
	memset(aesContext.ksch, '\0', 240);
	aes_set_key(appKey, 16, &aesContext);
	aes_encrypt(cryptbox, nwkSKey, &aesContext);
	//Decrypt appSkey
	cryptbox[0] = 0x02;
	memset(aesContext.ksch, '\0', 240);
	aes_set_key(appKey, 16, &aesContext);
	aes_encrypt(cryptbox, appSKey, &aesContext);
	return result;
}

//=== Function: LorawanJoin =============================================================================================
///
/// @brief	Join LoraWan Network via Join Request and Join Response Messenges
///
/// @param	uint8_t channel, uint8_t dataRate
///
/// @return bool
//=======================================================================================================================
bool LorawanJoinNetwork (uint8_t channel, uint8_t dataRate)
{	
	//Check Variable
	bool result;
	result = true;
	//Loop Variable
	uint8_t i = 0;
	//Join Request Array
	uint8_t joinRequest[23];
	//Join Accept Array
	uint8_t joinAccept[28];
	//Length of Join Accept
	uint8_t joinAcceptLength;
	//Change Channel
	LorawanChangeChannel(channel);
	//Change Datarate
	LorawanChangeDr(dataRate);
	//Compile Join Request
	LorawanCreateJoinRequest(&joinRequest[0]);
	//Transmit Join Request
	Rfm95Tx(&joinRequest[0],23);
	//Flip I/O for LoRa
	Rfm95WriteReg(0x33,0x67);
	//Idle for Receive Delay
	for(i=0;i<(joinAcceptDelay_1);i++)
	{
		Atmega328pStopWatchStart(1000);
		while(!Atmega328pStopWatchCheck());
	}
	//Start Stopwatch for second receive window
	Atmega328pStopWatchStart(1000);
	//Check for rx data (receive window 1) // rx timeout set for <1 sec with SF=12,  BW=125kHz
	if (Rfm95Rx(&joinAccept[0], (255-(dataRates[dataRate*2])*19), &joinAcceptLength))
	{	
		//Decrypt Join Accept and extract Network and Application Session Key
		LorawanJoinDecrypt(&joinAccept[1], (joinAcceptLength-1));
	}
	else
	{
		//Change Channel and DR for receive window 2
		Rfm95ChangeCarFreq(rx2Channel);
		LorawanChangeDr(rx2DataRate);
		//Wait for Stopwatch to end (receive window 2)
		while(!Atmega328pStopWatchCheck());
		//Check for rx
		if (Rfm95Rx(&joinAccept[0], (255-(dataRates[dataRate*2])*19) , &joinAcceptLength))
		{
			//Decrypt Join Accept and extract Network and Application Session Key
			LorawanJoinDecrypt(&joinAccept[1], (joinAcceptLength-1));
		}
		else
		{
			//Return false for no answer from network
			result = false;
		}
		LorawanChangeChannel(currentChannel);
		LorawanChangeDr(currentDataRate);
		
	}
	//Reset LoRa I/O flip
	Rfm95WriteReg(0x33,0x27);
	//Increase devNonce
	if (devNonce[1] == 0xFF)
	{
		if (devNonce[0] == 0xFF)
		{
			devNonce[0] = 0;
			devNonce[1] = 0;
		}
		else
		{
			devNonce[0]++;
		}
	}
	else
	{
		devNonce[1]++;
	}
	return result;
}



//=== Function: LorawanCreateMessage =============================================================================================
///
/// @brief	Compile Message for transmission via Lorawan
///
/// @param	uint8_t mType, uint8_t  *firstElementOfDevAddr, uint8_t fCtrl, uint32_t fCnt, uint8_t *fOpts, uint8_t fPort, uint8_t *firstElementOfPayload, uint8_t lengthOfPayload, uint8_t *firstElementOfOutput
///
/// @return bool
//=====================================================================================================================
bool LorawanCreateFrame(uint8_t mType, bool adr, bool adrAckReq, uint8_t fPort, uint8_t *firstElementOfOutput, uint8_t *lengthOfMessage)
{
	bool result = true;
	//Array for whole Message
	uint8_t message[127] = {0};
	//Encrypted Payload Array
	uint8_t payloadEnc[127];
	//MIC Array
	uint8_t messageMic[4];
	//Key Array
	uint8_t key[16];
	//Array Variable
	uint8_t i = 0;
	//Aux Variable
	uint8_t j = 0;
	//Aux Variable
	uint8_t k = 0;
	//fCtrl Byte
	uint8_t fCtrl;
	//Check for Pending MAC Command and its length
	uint8_t fOptsLen;
	if (macPending)
	{
		fOptsLen = macPendingLength;
		macPending = false;
	}
	else
	{
		fOptsLen = 0;
	}
	//Add mType to Message
	message[i] = (mType<<5);
	i++;
	k = i+4;
	//Signal pending ack request
	if (mType == CON_DATA_UP)
	{
		ackPendingUp = true;
	}
	//Add Device Address to Message
	while (i<k)
	{
		message[i] = devAdress[j];
		i++;
		j++;
	}
	//Add fCtrl to Message
	fCtrl = 0x00;
	if (adr)
	{
		fCtrl |= 0x80;
	}
	if (adrAckReq)
	{
		fCtrl |= 0x40;
	}
	if (ackPendingDown)
	{
		fCtrl |= 0x20;
	}
	fCtrl |= fOptsLen;
	message[i] = fCtrl;
	i++;
	//Add fCnt to Message
	message[i] = fCntUp;
	i++;
	message[i] = (fCntUp>>8);
	i++;
	//Add MAC Command (fOpts) to Message, length defined in fCtrl
	if (fOptsLen > 0)
	{
		if (fPort == 0)
		{
			fPort = 1;
		}
	}
	if (fPort == 0)
	{
		for (j=0; j<16; j++)
		{
			key[j] = nwkSKey[j];
		}
	}
	else
	{
		for (j=0; j<16; j++)
		{
			key[j] = appSKey[j];
		}

	}
	j = 0;
	k = i+fOptsLen;
	while (i<k)
	{
		message[i] = macPendingCommand[j];
		i++;
		j++;
	}
	//Add fPort to Message if lengthOfPayload < 0. 0 if Payload contains only MAC Commands
	if (!payloadPendingUp)
	{
		payloadPendingUpLength = 0;
	}
	//Add fPort to Message if lengthOfPayload < 0. 0 if Payload contains only MAC Commands
	if (payloadPendingUpLength > 0)
	{
		message[i] = fPort;
		i++;
		//Encrypt Payload if pending
		LorawanPayloadXCrypt(0x00, &devAdress[0], fCntUp, &key[0], &payloadPendingUpContent[0], payloadPendingUpLength, &payloadEnc[0]);
	}
	//Add Encrypted Payload to Message if pending
	j = 0;
	k = i+payloadPendingUpLength;
	while (i < k)
	{
		message[i] = payloadEnc[j];
		i++;
		j++;
	}
	//Calculate MIC
	LorawanMicCalc(0x00, &devAdress[0], fCntUp, &nwkSKey[0], &message[0], i, &messageMic[0]);
	//Add MIC to Message
	j = 0;
	k = i+4;
	while (i<k)
	{
		message[i] = messageMic[j];
		i++;
		j++;
	}
	*lengthOfMessage = i;
	j=0;
	while (j<i)
	{
		*(firstElementOfOutput+j) = message[j];
		j++;
	}
	return result;
} 

//=== Function: LorawanPayloadXCrypt =============================================================================================
///
/// @brief	Encrypt and Decrypt LoraWan Payload using AES128
///
/// @param	uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementofPayload, uint8_t lengthOfPayload, uint8_t *firstElementOfOutput
///
/// @return bool
//=====================================================================================================================
bool LorawanPayloadXCrypt (uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementofPayload, uint8_t lengthOfPayload, uint8_t *firstElementOfOutput)
{
	//Check Variable
	bool result = true;
	//A-Block Counter
	uint8_t i = 1;
	//Size of Payload
	uint8_t size = lengthOfPayload;
	//Array for A-Blocks
	uint8_t aBlock[16] = {0};
	//Array for S-Blocks
	uint8_t sBlock[16];
	//Loop Variable
	uint8_t j;
	//AES Context Variable
	aes_context aesContext;
	memset(aesContext.ksch, '\0', 240);
	//Set Key in Context Variable
	aes_set_key(key, 16, &aesContext);
	//Set A-Block
	aBlock[0] = 0x01;
	aBlock[5] = direction;
	aBlock[6] = *(firstElementOfDevAddr);
	aBlock[7] = *(firstElementOfDevAddr+1);
	aBlock[8] = *(firstElementOfDevAddr+2);
	aBlock[9] = *(firstElementOfDevAddr+3);
	aBlock[10] = fCnt;
	aBlock[11] = (fCnt>>8);
	aBlock[12] = (fCnt>>16);
	aBlock[13] = (fCnt>>24);
	//En-/Decryption Loop
	while (size >= 16)
	{
		aBlock[15] = i;
		i++;
		aes_encrypt(aBlock, sBlock, &aesContext);
		for (j=0; j<16; j++)
		{
			*(firstElementOfOutput+((i-1)*16+j)) = (*(firstElementofPayload+((i-1)*16+j)) ^ sBlock[j]);
		}
		size -= 16;
	}
	if (size>0)
	{
		aBlock[15] = i;
		aes_encrypt(aBlock, sBlock, &aesContext);
		for (j=0; j<size ; j++)
		{
			*(firstElementOfOutput+((i-1)*16+j)) = (*(firstElementofPayload+((i-1)*16+j)) ^ sBlock[j]);
		}
	}
	return result;
}
	
//=== Function: LorawanMicCalc =============================================================================================
///
/// @brief	Calculate MIC (Message Integrity Code) over whole Message via CMAC encryption
///
/// @param	uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementOfMessage, uint8_t lengthOfMessage, uint8_t *micOutput
///
/// @return bool
//=====================================================================================================================
bool LorawanMicCalc (uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementOfMessage, uint8_t lengthOfMessage, uint8_t *micOutput)
{
	//Check Variable
	bool result = true;
	//loop variable
	uint8_t i;
	//MIC calculation Block
	//uint8_t micBlock[16];
	uint8_t micBlock[16] = {0};
	//Calculated MIC field
	uint8_t mic[16];
	//CMAC Context Variable
	AES_CMAC_CTX aesCmacCtx[1];
	//Set MIC Calculation Block
	micBlock[0] = 0x49;
	micBlock[5] = direction;
	micBlock[6] = *(firstElementOfDevAddr);
	micBlock[7] = *(firstElementOfDevAddr+1);
	micBlock[8] = *(firstElementOfDevAddr+2);
	micBlock[9] = *(firstElementOfDevAddr+3);
	micBlock[10] = fCnt;
	micBlock[11] = (fCnt>>8);
	micBlock[12] = (fCnt>>16);
	micBlock[13] = (fCnt>>24);
	micBlock[15] = lengthOfMessage;
	for (i=0; i<lengthOfMessage; i++)
	{
		micBlock[(16+i)] = *(firstElementOfMessage+i);
	}
	//Initialize and Execute CMAC
	AES_CMAC_Init(aesCmacCtx);
	AES_CMAC_SetKey(aesCmacCtx, key);
	AES_CMAC_Update(aesCmacCtx, micBlock, (lengthOfMessage+16));
	AES_CMAC_Final(mic, aesCmacCtx);
	// Output MIC
	*micOutput = mic[0];
	*(micOutput+1) = mic[1];
	*(micOutput+2) = mic[2];
	*(micOutput+3) = mic[3];
	return result;
}
//=== Function: LorawanReadFrame =============================================================================================
///
/// @brief	Read and process LoRaWAN Frame, Output = Payload
///
/// @param	uint8_t *firstElementOfInput, uint8_t lengthOfInput
///
/// @return bool
//=======================================================================================================================
bool LorawanReadFrame(uint8_t *firstElementOfInput, uint8_t lengthOfInput)
{
	//Check Variable
	bool result = true;
	//Counter/Loop Variable
	uint8_t i = 0;
	//Aux Variable
	uint8_t j = 0;
	//Array for Encrypted Payload
	uint8_t encPayload[PAYLOAD_MAX];
	//Array for decrypted payload
	uint8_t decPayload[PAYLOAD_MAX];
	//Length of Payload
	uint8_t lengthOfPayload;
	//AES decryption Key
	uint8_t key[16];
	//mtype
	uint8_t mType;
	//Acknowledgement Bit
	bool ack;
	//fOpsLen
	uint8_t fOptsLen;
	//fOpts Array
	uint8_t fOpts[15];
	//fPort
	uint8_t fPort;
	//MIC
	uint8_t mic[4];
	//Calculated MIC
	uint8_t micCalc[4];
	//Only Continue if the device adress does match, otherwise frame is ignored
	if ((devAdress[0] == (*(firstElementOfInput+1)) && (devAdress[1] == *firstElementOfInput+2)) && (devAdress[2] == *(firstElementOfInput+3)) && (devAdress[3] == *(firstElementOfInput+4)))
	{	
		//Read the two LSB of the downlink frame counter from message
		fCntDown = ((fCntDown & 0xFFFFFF00) | (*(firstElementOfInput+6)));
		fCntDown = ((fCntDown & 0xFFFF00FF) | (*(firstElementOfInput+7)) << 8);
		if (fCntDown == fCntUp)
		{
			//Read mType from message
			mType = (((*firstElementOfInput) & 0xE0) >> 5);
			//Process FCtrl Byte
			//Read fOptsLen
			fOptsLen = (*(firstElementOfInput+5)) & 0x0F;
			//if fOptsLen > 0, MAC Commands are transmitted in fOpts, read fOpts	
			while (i<fOptsLen)
			{
				fOpts[i] = *(firstElementOfInput+8+i);
				i++;
			}
			//Set index to current position in the received message
			i =+ 8;
			//Read fPort 
			fPort = *(firstElementOfInput+i);
			i++;
			j= 0;
			//Depending on fPort, use nwkSKey or appSKey for decryption
			if (fPort == 0)
			{
				while (j<16)
				{
					key[j] = nwkSKey[j];
					j++;
				} 
			}
			else
			{
				while (j<16)
				{
					key[j] = appSKey[j];
					j++;
				}
			}
			j = 0;
			//If present, write encrypted payload to array (from current position to MIC field
			while (i<(lengthOfInput-4))
			{
				encPayload[j] = *(firstElementOfInput+i);
				j++;
				i++;
			}
			lengthOfPayload = (j+1);
			j=0;
			//Read MIC
			while (j<4)
			{
				mic[j] = *(firstElementOfInput+i);
				j++;
				i++;
			}
			//Calculate MIC of received Message
			LorawanMicCalc(1,&devAdress[0], fCntDown, &nwkSKey[0],firstElementOfInput,(i-5), &micCalc[0]);
			//Continue only if MIC is correct and fPort is different from 0 if foptsLen is > 0 , otherwise no payload gets forwarded and no MAC Command is executed
			if ((mic[0] == micCalc[0]) && (mic[1] == micCalc[1]) && (mic[2] == micCalc[2]) && (mic[3] == micCalc[3]) && ((fOptsLen > 0) != (fPort == 0)) )
			{
				//Set ackPendingDown flag if received message requirs acknowledgement from next uplink
				if (mType == CON_DATA_DOWN)
				{
					ackPendingDown = true;
				}
				//Read ADR and ACK Bit
				adr = 0xFF && ((*(firstElementOfInput+5)) & 0x80);
				ack = 0xFF && ((*(firstElementOfInput+5)) & 0x20);
				
				//UartPrintLine("ACK BIT in Downlink: ", &ack, 1);
				
				//If an ACK was pending from last Uplink and ACK in received Message is set, clear ackPendingUp flag
				if (ack && ackPendingUp)
				{
					ackPendingUp = false;
				}
				//Read fPending
				fPending = 0xFF && ((*(firstElementOfInput+5)) & 0x10);
				//If the two LSB of the downlink frame counter are 0xFF, increment the counter
				if (((*(firstElementOfInput+6)) & (*(firstElementOfInput+7))) == 0xFF)
				{
					fCntDown++;
				}
				//Decrypt and output Payload if fPort > 0
				if (fPort > 0)
				{
					LorawanPayloadXCrypt(1, &devAdress[0],fCntDown,&key[0], &encPayload[0], lengthOfPayload, &decPayload[0]);
					j=0;
					while (j<lengthOfPayload)
					{
						payloadPendingDownContent[j] = decPayload[j];
						j++;
						payloadPendingDownLength = lengthOfPayload;
					}
					//Set flag for pending downlink payload
					payloadPendingDown = true;
					
				}
				//Write Payload to fOpts if fPort = 0 and Process MAC Command (only MAC Commands in Payload for fPort = 0, not encrypted)
				if (fPort == 0)
				{
					if (lengthOfPayload )
					{
						j=0;
						while (j<lengthOfPayload)
						{
							fOpts[j] = decPayload[j];
						}
						
						//Insert MAC Command Function!!!
					}
				
					
				}
				//Process MAC Command in fOpts if fOptsLen > 0 (fPort cannot be 0 then)
				if (fOptsLen > 0)
				{
					//Insert MAC Command Function!!!
				}
			}
		}
		else
		{
			//UartPrintLine("Receive Gescheitert",0,0);
			//UartPrintNewLine();
			
			result = false;
		}
	}
	else
	{
		result = false;
	}
		
	
	return result;
}
//=== Function: LorawanSession =============================================================================================
///
/// @brief	Start Lorawan Session
///
/// @param
///
/// @return bool
//=======================================================================================================================
void LorawanStartSession()
{
	//Array for uplink frame
	uint8_t frameUp[PAYLOAD_MAX+28];
	//Length of Uplink frame
	uint8_t frameUpLength;
	//Array for downlink frame
	uint8_t frameDown[PAYLOAD_MAX+28];			
	//Length of downlink frame
	uint8_t frameDownLength;
	//Indicate transmitted message
	bool transCheck = false;
	//Reset frame counters
	fCntUp = 0;
	fCntDown = 0;
	//Counter variable
	uint16_t i = 0;
	//Retransmission Counter
	uint16_t reTrCnt = 0;
	//Set pseudo random value for retransmision back-off, increase if <100 (max 36 sec of transmission time per h if <1 sec per transmission for lowest data rate)
	reTransBackOff = Rfm95GetRandomNumber();
	if (reTransBackOff < 100)
	{
		reTransBackOff += Rfm95GetRandomNumber();
	}
	//Set current channel to channel 0
	currentChannel = 0;
	//Set current data rate to highest possible
	currentDataRate = (NUMBER_OF_DATA_RATES-1);
	
	UartPrintLine("Start Joining Network",0,0);
	UartPrintNewLine();
	
	//Join Network, start with first channel and max data rate
	//Execute only while Session is active
	while (!activeSession)
	{
		//Only Continue if join procedure was not successful
		if (!LorawanJoinNetwork(currentChannel,currentDataRate))
		{
			//Switch to next channel, decrease data rate if last channel is selected, reset both for last channel and minimal data rate selected
			if (currentChannel == (NUMBER_OF_CHANNELS-1))
			{
				if (currentDataRate == 0)
				{
					currentChannel = 0;
					currentDataRate = NUMBER_OF_DATA_RATES-1;
				}
				else
				{
					currentDataRate--;
					currentChannel = 0;
					
				}
			}
			else
			{
				currentChannel++;
			}
			//Wait for reTransBackOff seconds until new transmission
			for (i=0; i<reTransBackOff; i++)
			{
				Atmega328pStopWatchStart(1000);
				while (!Atmega328pStopWatchCheck());
			}
			//Increment retransmission counter
			reTrCnt++;
			//Increase reTransBackOff if reTrCnt reaches multiples of 36 (time per transmission <1 sec and <100 sec of reTransBackOff => <36 sec of transmission time in first h and <36 sec for following 10h
			if (reTrCnt == 36)
			{
				reTransBackOff *= 10;
			}
			else if (reTrCnt == 72)
			{
				reTransBackOff *= 10;
			}
			//Terminate Session if max number of retransmissions is reached
			else if (reTrCnt >= RETRAN_MAX)
			{
				return;
			}
		}
		//declare active session and reset retransmission parameters for successful join procedure
		else
		{
			activeSession = true;
			reTrCnt = 0;
			reTransBackOff = Rfm95GetRandomNumber();
			if (reTransBackOff < 100)
			{
				reTransBackOff += Rfm95GetRandomNumber();
			}
				
			UartPrintLine("Successfully joined Network",0,0);
			UartPrintNewLine();
			checkFreeStack();
			UartPrintLine("Wait" ,0,0);
			UartPrintNewLine();
			
			//Wait for reTransBackOff sec until first frame is allowed to be send
			for (i=0; i<reTransBackOff; i++)
			{
				Atmega328pStopWatchStart(1000);
				while (!Atmega328pStopWatchCheck());
			}
			
			
		}
	
	}
	//Execute while Session is active
	while (activeSession)
	{
		//Execute when payload is pending or a pending frame is indicated by last downlink message
		if (payloadPendingUp | fPending)
		{
			
			UartPrintNewLine("Payload or next Downlink Frame pending",0,0);
			UartPrintNewLine();
			
			transCheck = false;
			//Compile LoRaWAN Uplink frame	
			LorawanCreateFrame(MTYPE_DEFAULT,adr, 0, FPORT_DEFAULT, &frameUp[0], &frameUpLength);
			
			UartPrintLine("Compiled Frame: ", &frameUp[0],frameUpLength);
			UartPrintNewLine();
			checkFreeStack();
			
			//Erase payload and frame pending flag 
			//Payload pending flag is erased as soon as possible after compiling frame to not skip new payload
			//Frame is lost if max number of retransmission is reached and the session is terminated!
			payloadPendingUp = false;
			fPending = false;
			//Transmit and listen for answer form sever at least once, retransmit if acknowledgment was requested but not received
			while(ackPendingUp | !transCheck)	
			{

				UartPrintLine("Ack Pending for last Uplink",0,0);
				UartPrintNewLine();
				
				//Transmit Uplink Frame
				Rfm95Tx(&frameUp[0],frameUpLength);
				
				UartPrintLine("Send Frame: ", &frameUp[0], frameUpLength);
				UartPrintNewLine();
				checkFreeStack();	
				
				//Signal transmitted Uplink
				transCheck = true;
				//Flip I/0 for LoRa
				Rfm95WriteReg(0x33,0x67);
				//Wait for receiveDelay_1 seconds
				for (i=0; i<receiveDelay_1; i++)
				{
					Atmega328pStopWatchStart(1000);
					while(!Atmega328pStopWatchCheck());
				}
				Atmega328pStopWatchStart(1000);
				//Listen for answer in first receive window
				if (Rfm95Rx(&frameDown[0], (255-(dataRates[currentDataRate*2])*19), &frameDownLength))
				{
					
					UartPrintLine("Received Frame: ", &frameDown[0], frameDownLength);
					UartPrintNewLine();
					
					LorawanReadFrame(&frameDown[0], frameDownLength);
						
				}
				//Listen for answer in second receive window
				else
				{
					while(!Atmega328pStopWatchCheck())
					Rfm95ChangeCarFreq(RX2_FREQUENCY);
					LorawanChangeDr(rx2DataRate);
					if (Rfm95Rx(&frameDown[0], (255-(dataRates[currentDataRate*2])*19), &frameDownLength))
					{
						UartPrintLine("Received Frame: ", &frameDown[0], frameDownLength);
						UartPrintNewLine();
						
						LorawanReadFrame(&frameDown[0], frameDownLength);
						
					}
					LorawanChangeChannel(currentChannel);
					LorawanChangeDr(currentDataRate);
				}					
				//Reset LoRa I/O flip
				Rfm95WriteReg(0x33,0x27);
				
				checkFreeStack();
				
				//Wait for reTransBackOff sec until new transmission
				for (i=0; i<reTransBackOff; i++)
				{
					Atmega328pStopWatchStart(1000);
					while (!Atmega328pStopWatchCheck());
				}		
				//Increase reTransBackOff if reTrCnt reaches multiples of 36, terminate Session if reTrCnt reaches allowed maximum
				reTrCnt++;
				if (reTrCnt == 36)
				{
					reTransBackOff *= 10;
				}
				else if (reTrCnt == 72)
				{
					reTransBackOff *= 10;
				}
				else if (reTrCnt >= RETRAN_MAX)
				{
					activeSession = false;
					return;
				}		
			}
			//Increase Frame Counter
			fCntUp++;

			//Reset retransmission parameters
			reTrCnt = 0;
			reTransBackOff = Rfm95GetRandomNumber();
			if (reTransBackOff < 100)
			{
				reTransBackOff += Rfm95GetRandomNumber();
			}
		}
		
	}
	
	
}

