
//=====================================================================================================================
/// @file		lorawan.h
///
/// @brief		Variables and functions for transceiving data via LoRaWAN protocol 
///
/// @version	V1.0
///
/// @date		01.09.20
///
/// @author		Philipp Holle
//=====================================================================================================================
#ifndef LORAWAN_H_
#define LORAWAN_H_
//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "aes.h"
#include "cmac.h"
#include "RFM95W.h"
#include "atmega328p.h"
#include "UART_atmega328p.h"
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h> 

#include "STACKTEST.h"
//---------------------------------------------------------------------------------------------------------------------
// Defines
//---------------------------------------------------------------------------------------------------------------------
//MAC Command Identifiers (CID)
#define	LINKCHECK		0x02	//Up: Request validation of connectivity						//Down: Answer to validation request; contains reception quality estimation
#define	LINKADR			0x03	//Up: Acknowledge request to adjust data rate					//Down: Request to adjust data date
#define DUTYCYCLE		0x04	//Up: Acknowledge received maximum duty-cycle					//Down: Set maximum duty-cycle of device
#define RXPARAMSETUP	0x05	//Up: Acknowledge reception slot parameters						//Down: Set reception slot parameters
#define DEVSTATUS		0x06	//Up: Return end-device status (battery level, demod. margin)	//Down: Request status of end-device
#define NEWCHANNEL		0x07	//Up: Acknowledge new/altered radio channel						//Down: Creates or modifies definition of radio channel
#define RXTIMINGSETUP	0x08	//Up: Acknowledge reception slot timing							//Down: Set reception slot timing
#define TXPARAMSETUP	0x09	//Up: Acknowledge dwell time and max. EIRP						//Down: Set dwell time and max. EIRP
#define DLCHANNEL		0x0A	//UP: Acknowledge modification of downlink channel				//Down: Modify definition of downlink channel
#define DEVICETIME		0x0D	//Up: Request current date and time								//Down: Return current date and time
//Message Types
#define JOIN_REQ		0x00			//Join Request
#define JOIN_ACC		0x01			//Join Accept
#define UNCON_DATA_UP	0x02			//Uplink Data with no request for acknowledgment by server
#define UNCON_DATA_DOWN	0x03			//Downlink Data with no request for acknowledgment by node
#define	CON_DATA_UP		0x04			//Uplink Data with request for acknowledgment by server
#define CON_DATA_DOWN	0x05			//Downlink Data with request for acknowledgment by node

//Data Rates
#define DATA_RATES				0x0C, 0x07, 0x0B, 0x07 ,0x0A, 0x07, 0x09, 0x07, 0x08, 0x07, 0x07, 0x07		//List of Data Rates in Pairs SF+Bandwidth  // SF: 7=128, 8=256, 9=512, 10=1024, 11=2048, 12=4096
																											// Bandwidth: 0=7.8kHz, 1=10.4kHz, 2=15.6kHz, 3=20.8kHz, 4=31.25kHz, 5=41.7kHz, 6=62.5kHz, 7=125kHz, 8=250kHz, 9=500kHz
#define NUMBER_OF_DATA_RATES	6
#define DR_RX2					3																			
//Channels
#define CHANNEL_LIST			868100000, 868300000, 868500000, 		//Carrier Frequencies of Channels
#define NUMBER_OF_CHANNELS		3										//Number of Channels
#define RX2_FREQUENCY			869525000								//Default Carrier Frequency for RX2 in the 868MHz band
//Payload
#define PAYLOAD_MAX				51										//Max number of payload bytes //Max 51 @DR0!
//Default Port
#define FPORT_DEFAULT			0x01									//Default frame port
//Default Message Type
#define MTYPE_DEFAULT			0x04									//Default message type for uplink //Either unconfirmed (0x02) or confirmed(0x04)
//Adjust Data Rate
#define ADR_DEFAULT				1										//Activate/deactivate the adjust data rate option
//Max retransmissions
#define RETRAN_MAX				108										//Max number of retransmissions before new session gets started, max 2^16

//---------------------------------------------------------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------+
uint8_t devNonce[2];
//Payload pending for Uplink
bool payloadPendingUp;
uint8_t payloadPendingUpContent[PAYLOAD_MAX];
uint8_t payloadPendingUpLength;
//Payload pending for Downlink
bool payloadPendingDown;
uint8_t payloadPendingDownContent[51];
uint8_t payloadPendingDownLength;
//---------------------------------------------------------------------------------------------------------------------
// Prototypes of global functions
//---------------------------------------------------------------------------------------------------------------------
bool LorawanCreateFrame(uint8_t mType, bool adr, bool adrAckReq, uint8_t fPort, uint8_t *firstElementOfOutput, uint8_t *lengthOfMessage);
bool LorawanPayloadXCrypt (uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementofPayload, uint8_t lengthOfPayload, uint8_t *firstElementOfOutput);
bool LorawanMicCalc (uint8_t direction, uint8_t *firstElementOfDevAddr, uint32_t fCnt, uint8_t *key, uint8_t *firstElementOfMessage, uint8_t lengthOfMessage, uint8_t *micOutput);
bool LorawanJoinMicCalc (uint8_t *firstElementOfData, uint8_t *micOutput);
bool LorawanCreateJoinRequest(uint8_t *firstElementOfOutput);
bool LorawanJoinNetwork (uint8_t channel, uint8_t dataRate);
bool LorawanJoinDecrypt (uint8_t *firstElementOfData, uint8_t lengthOfMessage);
void LorawanStartSession();
#endif
